/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * filename: m-ecat72.c                                                    *
 *                                                                         *
 * UTIL C-source: Medical Image Conversion Utility                         *
 *                                                                         *
 * purpose      : Read ECAT 7.2 files                                      *
 *                                                                         *
 * project      : (X)MedCon by Erik Nolf                                   *
 *                                                                         *
 * Functions    : MdcCheckECAT7()          - Check for ECAT7 format        *
 *                MdcEcatPrintMainHdr()    - Print content main header     *
 *                MdcEcatPrintImgSubHdr()  - Print content image subheader *
 *                MdcReadECAT7()           - Read ECAT7 file               *
 *                MdcWriteECAT7()          - Write ECAT7 file              *
 *                                                                         *
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
/* $Id: m-ecat72.c,v 1.15 2003/06/04 23:12:16 enlf Exp $
 */

/*
   Copyright (C) 1997-2003 by Erik Nolf

   This program is free software; you can redistribute it and/or modify it
   under the terms of the GNU General Public License as published by the
   Free Software Foundation; either version 2, or (at your option) any later
   version.

   This program is distributed in the hope that it will be useful, but
   WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General
   Public License for more details.

   You should have received a copy of the GNU General Public License along
   with this program; if not, write to the Free Software Foundation, Inc.,
   59 Place - Suite 330, Boston, MA 02111-1307, USA.  */

/****************************************************************************
                              H E A D E R S
****************************************************************************/

#include <time.h>

#include "medcon.h"

/****************************************************************************
                              D E F I N E S
****************************************************************************/

#define MDC_MAX_ECAT7FILETYPES      15
#define MDC_MAX_ECAT7FILETYPES_SIZE 15

static char MdcEcat7FileTypes
        [MDC_MAX_ECAT7FILETYPES][MDC_MAX_ECAT7FILETYPES_SIZE]=
                       {"Unknown","Sinogram","Image16","AttnCorr","Norm",
                        "PolarMap","Volume8","Volume16","Projection8",
                        "Projection16","Image8","3DSino16","3DSino8",
                        "3DNorm","3DSinoFlt"};

/****************************************************************************
                            F U N C T I O N S
****************************************************************************/
int MdcCheckECAT7(FILEINFO *fi)
{
  Mdc_Main_header7 mh;

  if (mdc_mat_read_main_header7(fi->ifp,&mh)) return MDC_BAD_READ;

  if (memcmp(mh.magic_number,MDC_ECAT7_SIG,7) ) return(MDC_FRMT_NONE);

  return MDC_FRMT_ECAT7;

}

void MdcEcatPrintMainHdr(Mdc_Main_header7 *mh)
{
  int i;

  MdcPrintLine('-',MDC_HALF_LENGTH);
  MdcPrntScrn("ECAT7 Main Header (%d bytes)\n",MH_72_SIZE);
  MdcPrintLine('-',MDC_HALF_LENGTH);

  MdcGetSafeString(mdcbufr,(char *)mh->magic_number,14,14);
  MdcPrntScrn("magic_number             : %s\n",mdcbufr);
  MdcGetSafeString(mdcbufr,mh->original_file_name,32,32);
  MdcPrntScrn("original_file_name       : %s\n",mdcbufr);
  MdcPrntScrn("sw_version               : %hd\n",mh->sw_version);
  MdcPrntScrn("system_type              : %hd\n",mh->system_type);
  MdcPrntScrn("file_type                : %hd ",mh->file_type);
  if ((mh->file_type > -1) && (mh->file_type < 15))
    MdcPrntScrn("(= %s)\n",MdcEcat7FileTypes[mh->file_type]);
  else
    MdcPrntScrn("(= Unknown)\n");
  MdcGetSafeString(mdcbufr,mh->serial_number,10,10);
  MdcPrntScrn("serial_number            : %s\n",mdcbufr);
  MdcPrntScrn("scan_start_time          : %s\n"
                                        ,ctime((time_t *)&mh->scan_start_time));
  MdcGetSafeString(mdcbufr,mh->isotope_name,8,8);
  MdcPrntScrn("isotope_name             : %s\n",mdcbufr);
  MdcPrntScrn("isotope_halflife         : %f [sec]\n",mh->isotope_halflife);
  MdcGetSafeString(mdcbufr,mh->radiopharmaceutical,32,32);
  MdcPrntScrn("radiopharmaceutical      : %s\n",mdcbufr);
  MdcPrntScrn("gantry_tilt              : %f [degrees]\n",mh->gantry_tilt);
  MdcPrntScrn("gantry_rotation          : %f [degrees]\n",mh->gantry_rotation);
  MdcPrntScrn("bed_elevation            : %f [cm]\n",mh->bed_elevation);
  MdcPrntScrn("intrinsic_tilt           : %f [degrees]\n",mh->intrinsic_tilt);
  MdcPrntScrn("wobble_speed             : %hd [rpm]\n",mh->wobble_speed);
  MdcPrntScrn("tansm_source_type        : %hd\n",mh->transm_source_type);
  MdcPrntScrn("distance_scanned         : %f [cm]\n",mh->distance_scanned);
  MdcPrntScrn("transaxial_fov           : %f [cm]\n",mh->transaxial_fov);
  MdcPrntScrn("angular_compression      : %hd\n",mh->angular_compression);
  MdcPrntScrn("coin_samp_mode           : %hd\n",mh->coin_samp_mode);
  MdcPrntScrn("axial_samp_mode          : %hd\n",mh->axial_samp_mode);
  MdcPrntScrn("ecat_calibration_factor  : %e\n",mh->ecat_calibration_factor);
  MdcPrntScrn("calibration_units        : %hd\n",mh->calibration_units);
  MdcPrntScrn("calibration_units_label  : %hd\n",mh->calibration_units_label);
  MdcPrntScrn("compression_code         : %hd\n",mh->compression_code);
  MdcGetSafeString(mdcbufr,mh->study_type,14,14);
  MdcPrntScrn("study_type               : %s\n",mdcbufr);
  MdcGetSafeString(mdcbufr,mh->patient_id,16,16);
  MdcPrntScrn("patient_id               : %s\n",mdcbufr);
  MdcGetSafeString(mdcbufr,mh->patient_name,32,32);
  MdcPrntScrn("patient_name             : %s\n",mdcbufr);
  MdcPrntScrn("patient_sex              : ");
    switch (mh->patient_sex[0]) {
      case 0:  MdcPrntScrn("M\n"); break;
      case 1:  MdcPrntScrn("F\n"); break;
      default: MdcPrntScrn("U\n");
    }
  MdcPrntScrn("patient_dexterity        : %c\n",mh->patient_dexterity[0]);
  MdcPrntScrn("patient_age              : %f\n",mh->patient_age);
  MdcPrntScrn("patient_height           : %f\n",mh->patient_height);
  MdcPrntScrn("patient_weight           : %f\n",mh->patient_weight);
  MdcPrntScrn("patient_birth_date       : %f\n",mh->patient_birth_date);
  MdcGetSafeString(mdcbufr,mh->physician_name,32,32);
  MdcPrntScrn("physician_name           : %s\n",mdcbufr);
  MdcGetSafeString(mdcbufr,mh->operator_name,32,32);
  MdcPrntScrn("operator_name            : %s\n",mdcbufr);
  MdcGetSafeString(mdcbufr,mh->study_description,32,32);
  MdcPrntScrn("study_description        : %s\n",mdcbufr);
  MdcPrntScrn("acquisition_type         : %hd\n",mh->acquisition_type);
  MdcPrntScrn("patient_orientation      : %hd\n",mh->patient_orientation);
  MdcGetSafeString(mdcbufr,mh->facility_name,20,20);
  MdcPrntScrn("facility_name            : %s\n",mdcbufr);
  MdcPrntScrn("num_planes               : %hd\n",mh->num_planes);
  MdcPrntScrn("num_frames               : %hd\n",mh->num_frames);
  MdcPrntScrn("num_gates                : %hd\n",mh->num_gates);
  MdcPrntScrn("num_bed_pos              : %hd\n",mh->num_bed_pos);
  MdcPrntScrn("init_bed_position        : %f\n",mh->init_bed_position);
    for (i=0; i<15; i++) 
  MdcPrntScrn("bed_position[%2d]         : %f\n",i,mh->bed_position[i]);
  MdcPrntScrn("plane_separation         : %f [cm]\n",mh->plane_separation);
  MdcPrntScrn("lwr_sctr_thres           : %hd [Kev]\n",mh->lwr_sctr_thres);
  MdcPrntScrn("lwr_true_thres           : %hd [Kev]\n",mh->lwr_true_thres);
  MdcPrntScrn("upr_true_thres           : %hd [Kev]\n",mh->upr_true_thres);
  MdcGetSafeString(mdcbufr,mh->user_process_code,10,10);
  MdcPrntScrn("user_process_code        : %s\n",mdcbufr);
  MdcPrntScrn("acquisition_mode         : %hd\n",mh->acquisition_mode);
  MdcPrntScrn("bin_size                 : %f [cm]\n",mh->bin_size);
  MdcPrntScrn("branching_fraction       : %f\n",mh->branching_fraction);
  MdcPrntScrn("dose_start_time          : %s\n"
                                        ,ctime((time_t *)&mh->dose_start_time));
  MdcPrntScrn("dosage                   : %e [mCi]\n",mh->dosage);
  MdcPrntScrn("well_counter_corr_factor : %f\n",mh->well_counter_corr_factor);
  MdcGetSafeString(mdcbufr,mh->data_units,32,32);
  MdcPrntScrn("data_units               : %s\n",mdcbufr);
  MdcPrntScrn("septa_state              : %hd\n",mh->septa_state);
    for (i=0; i<6; i++) 
  MdcPrntScrn("fill_cti[%d]              : %hd\n",i,mh->fill_cti[i]);

}

void MdcEcatPrintImgSubHdr(Mdc_Image_subheader7 *ish, int nr)
{
  MdcPrintLine('-',MDC_HALF_LENGTH);
  MdcPrntScrn("ECAT7 Image Sub Header %05d (%d bytes)\n",nr,ISH_72_SIZE);
  MdcPrintLine('-',MDC_HALF_LENGTH);

  MdcPrntScrn("data_type                : %hd\n",ish->data_type);
  MdcPrntScrn("num_dimensions           : %hd\n",ish->num_dimensions);
  MdcPrntScrn("x_dimension              : %hd\n",ish->x_dimension);
  MdcPrntScrn("y_dimension              : %hd\n",ish->y_dimension);
  MdcPrntScrn("z_dimension              : %hd\n",ish->z_dimension);
  MdcPrntScrn("x_offset                 : %f [cm]\n",ish->x_offset);
  MdcPrntScrn("y_offset                 : %f [cm]\n",ish->y_offset);
  MdcPrntScrn("z_offset                 : %f [cm]\n",ish->z_offset);
  MdcPrntScrn("recon_zoom               : %f\n",ish->recon_zoom);
  MdcPrntScrn("scale_factor             : %e\n",ish->scale_factor);
  MdcPrntScrn("image_min                : %hd\n",ish->image_min);
  MdcPrntScrn("image_max                : %hd\n",ish->image_max);
  MdcPrntScrn("x_pixel_size             : %f [cm]\n",ish->x_pixel_size);
  MdcPrntScrn("y_pixel_size             : %f [cm]\n",ish->y_pixel_size);
  MdcPrntScrn("z_pixel_size             : %f [cm]\n",ish->z_pixel_size);
  MdcPrntScrn("frame_duration           : %d [msec]\n",ish->frame_duration);
  MdcPrntScrn("frame_start_time         : %d [msec]\n",ish->frame_start_time);
  MdcPrntScrn("filter_code              : %hd\n",ish->filter_code);
  MdcPrntScrn("x_resolution             : %g\n",ish->x_resolution);
  MdcPrntScrn("y_resolution             : %g\n",ish->y_resolution);
  MdcPrntScrn("z_resolution             : %g\n",ish->z_resolution);
  MdcPrntScrn("num_r_elements           : %g\n",ish->num_r_elements);
  MdcPrntScrn("num_angles               : %g\n",ish->num_angles);
  MdcPrntScrn("z_rotation_angle         ; %g\n",ish->z_rotation_angle);
  MdcPrntScrn("decay_corr_fctr          : %g\n",ish->decay_corr_fctr);
  MdcPrntScrn("processing_code          : %d\n",ish->processing_code);
  MdcPrntScrn("gate_duration            : %u\n",ish->gate_duration);
  MdcPrntScrn("r_wave_offset            : %d\n",ish->r_wave_offset);
  MdcPrntScrn("num_accepted_beats       : %d\n",ish->num_accepted_beats);
  MdcPrntScrn("filter_cutoff_frequency  : %g\n",ish->filter_cutoff_frequency);
  MdcPrntScrn("filter_resolution        : %g\n",ish->filter_resolution);
  MdcPrntScrn("filter_ramp_slope        : %g\n",ish->filter_ramp_slope);
  MdcPrntScrn("filter_order             : %hd\n",ish->filter_order);
  MdcPrntScrn("filter_scatter_fraction  : %g\n",ish->filter_scatter_fraction);
  MdcPrntScrn("filter_scatter_slope     : %g\n",ish->filter_scatter_slope);
  MdcGetSafeString(mdcbufr,ish->annotation,40,40);
  MdcPrntScrn("annotation               : %s\n",mdcbufr);
  MdcPrntScrn("mt_1_1                   : %g\n",ish->mt_1_1);
  MdcPrntScrn("mt_1_2                   : %g\n",ish->mt_1_2);
  MdcPrntScrn("mt_1_3                   : %g\n",ish->mt_1_3);
  MdcPrntScrn("mt_2_1                   : %g\n",ish->mt_2_1);
  MdcPrntScrn("mt_2_2                   : %g\n",ish->mt_2_2);
  MdcPrntScrn("mt_2_3                   : %g\n",ish->mt_2_3);
  MdcPrntScrn("mt_3_1                   : %g\n",ish->mt_3_1);
  MdcPrntScrn("mt_3_2                   : %g\n",ish->mt_3_2);
  MdcPrntScrn("mt_3_3                   : %g\n",ish->mt_3_3);
  MdcPrntScrn("rfilter_cutoff           : %g\n",ish->rfilter_cutoff);
  MdcPrntScrn("rfilter_resolution       : %g\n",ish->rfilter_resolution);
  MdcPrntScrn("rfilter_code             : %hd\n",ish->rfilter_code);
  MdcPrntScrn("rfilter_order            : %hd\n",ish->rfilter_order);
  MdcPrntScrn("zfilter_cutoff           : %g\n",ish->zfilter_cutoff);
  MdcPrntScrn("zfilter_resolution       : %g\n",ish->zfilter_resolution);
  MdcPrntScrn("zfilter_code             : %hd\n",ish->zfilter_code);
  MdcPrntScrn("zfilter_order            : %hd\n",ish->zfilter_order);
  MdcPrntScrn("mt_1_4                   : %g\n",ish->mt_1_4);
  MdcPrntScrn("mt_2_4                   : %g\n",ish->mt_2_4);
  MdcPrntScrn("mt_3_4                   : %g\n",ish->mt_3_4);
  MdcPrntScrn("scatter_type             : %hd\n",ish->scatter_type);
  MdcPrntScrn("recon_type               : %hd\n",ish->recon_type);
  MdcPrntScrn("recon_views              : %hd\n",ish->recon_views);
  MdcPrntScrn("fill_cti[87]             : <not printed>\n");
  MdcPrntScrn("fill_user[48]            : <not printed>\n"); 

}

void MdcEcatPrintAttnSubHdr(Mdc_Attn_subheader7 *ash)
{
  int i;

  MdcPrntScrn("data_type                : %hd\n",ash->data_type);
  MdcPrntScrn("num_dimensions           : %hd\n",ash->num_dimensions);
  MdcPrntScrn("attenuation_type         : %hd\n",ash->attenuation_type);
  MdcPrntScrn("num_r_elements           : %hd\n",ash->num_r_elements);
  MdcPrntScrn("num_angles               : %hd\n",ash->num_angles);
  MdcPrntScrn("num_z_elements           : %hd\n",ash->num_z_elements);
  MdcPrntScrn("ring_difference          : %hd\n",ash->ring_difference);
  MdcPrntScrn("x_resolution             : %g [cm]\n",ash->x_resolution);
  MdcPrntScrn("y_resolution             : %g [cm]\n",ash->y_resolution);
  MdcPrntScrn("z_resolution             : %g [cm]\n",ash->z_resolution);
  MdcPrntScrn("w_resolution             : %g\n",ash->w_resolution);
  MdcPrntScrn("scale_factor             : %e\n",ash->scale_factor);
  MdcPrntScrn("x_offset                 : %g [cm]\n",ash->x_offset);
  MdcPrntScrn("y_offset                 : %g [cm]\n",ash->y_offset);
  MdcPrntScrn("x_radius                 : %g [cm]\n",ash->x_radius);
  MdcPrntScrn("y_radius                 : %g [cm]\n",ash->y_radius);
  MdcPrntScrn("tilt_angle               : %g [degrees]\n",ash->tilt_angle);
  MdcPrntScrn("attenuation_coeff        : %g [1/cm]\n",ash->attenuation_coeff);
  MdcPrntScrn("attenuation_min          : %g\n",ash->attenuation_min);
  MdcPrntScrn("attenuation_max          : %g\n",ash->attenuation_max);
  MdcPrntScrn("skull_thickness          : %g [cm]\n",ash->skull_thickness);
  MdcPrntScrn("num_additional_atten_coeff: %hd\n"
                                        ,ash->num_additional_atten_coeff);
  for (i=0; i<8; i++)
  MdcPrntScrn("additional_atten_coeff[%d]: %g\n",i
                                        ,ash->additional_atten_coeff[i]);
  MdcPrntScrn("edge_finding_threshold   : %g\n",ash->edge_finding_threshold);
  MdcPrntScrn("storage_order            : %hd\n",ash->storage_order);
  MdcPrntScrn("span                     : %hd\n",ash->span);
  MdcPrntScrn("z_elements[64]           : <not printed>\n");
  MdcPrntScrn("full_unused[86]          : <not printed>\n");
  MdcPrntScrn("fill_user[50]            : <not printed>\n");

}

void MdcEcatPrintScanSubHdr(Mdc_Scan_subheader7 *ssh)
{
  MdcPrntScrn("data_type                : %hd\n",ssh->data_type);
  MdcPrntScrn("num_dimensions           : %hd\n",ssh->num_dimensions);
  MdcPrntScrn("num_r_elements           : %hd\n",ssh->num_r_elements);
  MdcPrntScrn("num_angles               : %hd\n",ssh->num_angles);
  MdcPrntScrn("corrections_applied      : %hd\n",ssh->corrections_applied);
  MdcPrntScrn("num_z_elements           : %hd\n",ssh->num_z_elements);
  MdcPrntScrn("ring_difference          : %hd\n",ssh->ring_difference);
  MdcPrntScrn("x_resolution             : %g [cm]\n",ssh->x_resolution);
  MdcPrntScrn("y_resolution             : %g [cm]\n",ssh->y_resolution);
  MdcPrntScrn("z_resolution             : %g [cm]\n",ssh->z_resolution);
  MdcPrntScrn("w_resolution             : %g\n",ssh->w_resolution);
  MdcPrntScrn("fill[6]                  : <not printed>\n");
  MdcPrntScrn("gate_duration            : %u [msec]\n",ssh->gate_duration);
  MdcPrntScrn("r_wave_offset            : %d [msec]\n",ssh->r_wave_offset);
  MdcPrntScrn("num_accepted_beats       : %d\n",ssh->num_accepted_beats);
  MdcPrntScrn("scale_factor             : %e\n",ssh->scale_factor);
  MdcPrntScrn("scan_min                 : %hd\n",ssh->scan_min);
  MdcPrntScrn("scan_max                 : %hd\n",ssh->scan_max);
  MdcPrntScrn("prompts                  : %d\n",ssh->prompts);
  MdcPrntScrn("delayed                  : %d\n",ssh->delayed);
  MdcPrntScrn("multiples                : %d\n",ssh->multiples);
  MdcPrntScrn("net_trues                : %d\n",ssh->net_trues);
  MdcPrntScrn("cor_singles[16]          : <not printed>\n"); 
  MdcPrntScrn("uncor_singles[16]        : <not printed>\n");
  MdcPrntScrn("tot_avg_cor              : %g\n",ssh->tot_avg_cor);
  MdcPrntScrn("tot_avg_uncor            : %g\n",ssh->tot_avg_uncor);
  MdcPrntScrn("total_coin_rate          : %d\n",ssh->total_coin_rate);
  MdcPrntScrn("frame_start_time         : %u\n",ssh->frame_start_time);
  MdcPrntScrn("frame_duration           : %u\n",ssh->frame_duration);
  MdcPrntScrn("deadtime_correction_factor: %g\n"
                                        ,ssh->deadtime_correction_factor);
  MdcPrntScrn("phy_planes[8]            : <not printed>\n");
  MdcPrntScrn("cti_fill[90]             : <not printed>\n");
  MdcPrntScrn("user_fill[50]            : <not printed>\n");

}


void MdcEcatPrintNormSubHdr(Mdc_Norm_subheader7 *nsh)
{
  MdcPrntScrn("data_type                : %hd\n",nsh->data_type);
  MdcPrntScrn("num_dimensions           : %hd\n",nsh->num_dimensions);
  MdcPrntScrn("num_r_elements           : %hd\n",nsh->num_r_elements);
  MdcPrntScrn("num_angles               : %hd\n",nsh->num_angles);
  MdcPrntScrn("num_z_elements           : %hd\n",nsh->num_z_elements);
  MdcPrntScrn("ring_difference          : %hd\n",nsh->ring_difference);
  MdcPrntScrn("scale_factor             : %e\n",nsh->scale_factor);
  MdcPrntScrn("norm_min                 : %g\n",nsh->norm_min);
  MdcPrntScrn("norm_max                 : %g\n",nsh->norm_max);
  MdcPrntScrn("fov_source_width         : %g\n",nsh->fov_source_width);
  MdcPrntScrn("norm_quality_factor      : %g\n",nsh->norm_quality_factor);
  MdcPrntScrn("norm_quality_factor_code : %hd\n",nsh->norm_quality_factor_code);
  MdcPrntScrn("storage_order            : %hd\n",nsh->storage_order);
  MdcPrntScrn("span                     : %hd\n",nsh->span);
  MdcPrntScrn("z_elements[64]           : <not printed>\n");
  MdcPrntScrn("cti_fill[123]            : <not printed>\n");
  MdcPrntScrn("user_fill[50]            : <not printed>\n");

}

char *MdcReadECAT7(FILEINFO *fi)
{
  FILE *fp = fi->ifp;
  int i, error, UNSUPPORTED;
  char *err, *str;
  struct tm *time;
  Mdc_Main_header7 mh;
  Mdc_Image_subheader7 ish;
/*
  Mdc_Scan_subheader7 ssh;
  Mdc_Attn_subheader7 ash;
  Mdc_Norm_subheader7 nsh;
*/
  struct Mdc_MatDir entry, matrix_list[MDC_ECAT7_MAX_MATRICES];
  struct Mdc_Matval matval;
  Uint32 number, slice, img=0, vol=0, bytes;
  int bed,gate,frame,plane,nb,ng,nf,np,nd;
  int matnum, startblk, endblk, num_matrices; 
  IMG_DATA *id;
  Int16 bits, type;
  Uint8 *mbufr;


#if XSUPPORTED
  if (XMDC_MEDCON) XMdcBeginProgressBar("Reading ECAT7:");
#endif 

  if (MDC_VERBOSE) MdcPrntMesg("ECAT7 Reading <%s> ...",fi->ifname);

  /* put some defaults we use */
  fi->endian=MDC_FILE_ENDIAN=MDC_BIG_ENDIAN;

  error = mdc_mat_read_main_header7(fp, &mh);
  if (error) return("ECAT7 Bad read main header");

  if (MDC_INFO) MdcEcatPrintMainHdr(&mh);

  /* check for supported file types */
  switch (mh.file_type) {
    case MDC_ECAT7_FILE_TYPE_IMAGE16 :
    case MDC_ECAT7_FILE_TYPE_VOLUME8 :
    case MDC_ECAT7_FILE_TYPE_VOLUME16:
    case MDC_ECAT7_FILE_TYPE_IMAGE8  :
        UNSUPPORTED = MDC_NO;
        break;
    default:
        UNSUPPORTED = MDC_YES;
  }
  if (UNSUPPORTED == MDC_YES) return("ECAT7 Unsupported file type");

  if (mh.num_frames  <= 0 ) mh.num_frames  = 1;
  if (mh.num_gates   <= 0 ) mh.num_gates   = 1;
  if (mh.num_bed_pos <  0 ) mh.num_bed_pos = 0;

  /* fill in global FILEINFO data */
  fi->dim[0]= 6;
  fi->dim[3]= mh.num_planes;
  fi->dim[4]= mh.num_frames;
  fi->dim[5]= mh.num_gates;
  fi->dim[6]= mh.num_bed_pos + 1; /* must be 1-based */

#if MDC_NUM_BEDS_TWEAK
  /* double check num_bed_pos value due to */
  /* inconsistent use as zero or one based */
  bed = mh.num_bed_pos;
  while (!mdc_mat_lookup(fp,mdc_mat_numcod(1,1,1,0,bed),&entry) && (bed > 0)) {
       bed--;
  }
  fi->dim[6] = bed + 1;
#endif

  /* check for unsupported bed overlap */
  if (fi->dim[6] > 1) {
    float axial_width, bed_offset=mh.bed_position[0];

    if (bed_offset < 0) bed_offset = -bed_offset;

    axial_width = mh.plane_separation * (float)fi->dim[3];
    if ((axial_width - bed_offset) >= 1.0) {
#if XSUPPORTED
      if (XMDC_MEDCON) {
        XMdcDisplayWarn("ECAT7 Bed overlaps unsupported");
      }else
#endif
      {
      MdcPrntWarn("ECAT7 Bed overlaps unsupported"); 
      }
    }
  }

  for (i=3, number=1; i<=6; i++) number*=fi->dim[i];

  if (number == 0) return("ECAT7 No valid images specified");

  /* fill in orientation information */
  switch (mh.patient_orientation) {
    case MDC_ECAT7_FEETFIRST_PRONE:
        fi->pat_slice_orient = MDC_PRONE_FEETFIRST_TRANSAXIAL;  break;
    case MDC_ECAT7_HEADFIRST_PRONE:
        fi->pat_slice_orient = MDC_PRONE_HEADFIRST_TRANSAXIAL;  break;
    case MDC_ECAT7_FEETFIRST_SUPINE:
        fi->pat_slice_orient = MDC_SUPINE_FEETFIRST_TRANSAXIAL; break;
    case MDC_ECAT7_HEADFIRST_SUPINE:
        fi->pat_slice_orient = MDC_SUPINE_HEADFIRST_TRANSAXIAL; break;
    default:
        fi->pat_slice_orient = MDC_SUPINE_HEADFIRST_TRANSAXIAL;
#if XSUPPORTED
        if (XMDC_MEDCON) {
          XMdcDisplayWarn("ECAT7 unknown patient orientation");
      }else
#endif
      {
          MdcPrntWarn("ECAT7 unknown patient orientation"); 
      }
  }

  str = MdcGetStrPatientPos(fi->pat_slice_orient);
  MdcStringCopy(fi->pat_pos,str,strlen(str));

  str = MdcGetStrPatientOrient(fi->pat_slice_orient);
  MdcStringCopy(fi->pat_orient,str,strlen(str));

  /* fill in patient study related information */
  fi->patient_sex[0] = mh.patient_sex[0]; fi->patient_sex[1]='\0';
  MdcStringCopy(fi->patient_name,mh.patient_name,32);
  MdcStringCopy(fi->patient_id,mh.patient_id,16);
  fi->patient_weight = mh.patient_weight;
  time = localtime((time_t *)&(mh.scan_start_time));
  fi->study_date_day   = time->tm_mday;
  fi->study_date_month = time->tm_mon  + 1;
  fi->study_date_year  = time->tm_year + 1900;
  fi->study_time_hour  = time->tm_hour;
  fi->study_time_minute= time->tm_min;
  fi->study_time_second= time->tm_sec;

  switch (mh.acquisition_type) {
    case MDC_ECAT7_SCAN_TRANSMISSION:
    case MDC_ECAT7_SCAN_STATIC_EMISSION:
      fi->acquisition_type = MDC_ACQUISITION_TOMO;    
      break;
    case MDC_ECAT7_SCAN_DYNAMIC_EMISSION:
      fi->acquisition_type = MDC_ACQUISITION_DYNAMIC; 
      break;
    case MDC_ECAT7_SCAN_GATED_EMISSION:
      fi->acquisition_type = MDC_ACQUISITION_GSPECT; 
      break;
    case MDC_ECAT7_SCAN_BLANK:
    case MDC_ECAT7_SCAN_TRANS_RECTILINEAR:
    case MDC_ECAT7_SCAN_EMISSION_RECTILINEAR:
    default: 
      fi->acquisition_type = MDC_ACQUISITION_UNKNOWN;
      break;
  }

  MdcStringCopy(fi->study_descr,mh.study_description,32);
  MdcStringCopy(fi->study_id,mh.study_type,12);
  MdcStringCopy(fi->radiopharma,mh.radiopharmaceutical,32);
  MdcStringCopy(fi->isotope_code,mh.isotope_name,8);

  fi->isotope_halflife = mh.isotope_halflife;
  fi->injected_dose    = MdcmCi2MBq(mh.dosage);
  fi->gantry_tilt      = mh.gantry_tilt;

  if (MDC_ECHO_ALIAS == MDC_YES) {
    MdcEchoAliasName(fi); return(NULL);
  }

  if (!MdcGetStructID(fi,number))
    return("ECAT7 Bad malloc IMG_DATA structs");

  /* ECAT7: matrices for each volume */
  num_matrices = mdc_mat_list7(fp, matrix_list, MDC_ECAT7_MAX_MATRICES);
  if (num_matrices == 0)
    return("ECAT7 No matrices found");
  if ((Uint32)num_matrices > (fi->number / fi->dim[3]))
    return("ECAT7 Too many matrices found");

  if (MDC_MY_DEBUG) {
    int t;

    printf("DEBUG: %d.%d.%d.%d\n\n",fi->dim[3]
                                   ,fi->dim[4]
                                   ,fi->dim[5]
                                   ,fi->dim[6]);

    for (t=0; t<num_matrices; t++) {
       printf("DEBUG: [%2d]\t%d\t%d\t%d\t%d\n",t
                                              ,matrix_list[t].matnum
                                              ,matrix_list[t].strtblk
                                              ,matrix_list[t].endblk
                                              ,matrix_list[t].matstat);
    }
  } 

  img=0; vol=0;
  for (bed=0; bed<fi->dim[6]; bed++)
  for (gate=1; gate<=fi->dim[5]; gate++)
  for (frame=1; frame<=fi->dim[4]; frame++, vol++)
  for (plane=1; plane<=fi->dim[3]; ) {

     if (vol == num_matrices) break;

     mdc_mat_numdoc(matrix_list[vol].matnum,&matval);
     nf = matval.frame; np = matval.plane; ng = matval.gate;
     nb = matval.bed;   nd = matval.data;
     matnum = mdc_mat_numcod(nf,np,ng,nd,nb);
     MdcDebugPrint("DEBUG: %d\n",matnum);
     if (!mdc_mat_lookup7(fp, matnum, &entry)) continue;
     startblk = entry.strtblk + 1;
     endblk = entry.endblk - entry.strtblk;

     MdcDebugPrint("DEBUG: entry.endblk = %d\n",entry.endblk);
     MdcDebugPrint("DEBUG: entry.strtblk= %d\n",entry.strtblk);
     MdcDebugPrint("DEBUG: startblk     = %d\n",startblk);
     MdcDebugPrint("DEBUG: endblk       = %d\n\n",endblk);

     switch (mh.file_type) {

       case MDC_ECAT7_FILE_TYPE_IMAGE8  :
       case MDC_ECAT7_FILE_TYPE_IMAGE16 :
       case MDC_ECAT7_FILE_TYPE_VOLUME8 :
       case MDC_ECAT7_FILE_TYPE_VOLUME16:

         error = mdc_mat_read_image_subheader7(fp, startblk-1, &ish);
         if (error) return("ECAT7 Bad read image subheader");

         if (MDC_INFO) MdcEcatPrintImgSubHdr(&ish, vol + 1);

         fi->dim[1] = ish.x_dimension;
         fi->dim[2] = ish.y_dimension;

         break;

       default:

         return("ECAT7 Unsupported file type");
     }

     switch (ish.data_type) {
       case BYTE_TYPE:
           bits = 8;  type = BIT8_U;  break;
       case VAX_I2:
       case SUN_I2:
           bits = 16; type = BIT16_S; break;
       case VAX_I4:
       case SUN_I4:
           bits = 32; type = BIT32_S; break;
       case VAX_R4:
       case IEEE_R4:
           bits = 32; type = FLT32;   break;
       default:
           return("ECAT7: Unsupported data type");
     }

     /* bytes entire volume (matrix blocks) */
     bytes = fi->dim[1] * fi->dim[2] * fi->dim[3] * MdcType2Bytes(type);
     MdcDebugPrint("DEBUG: bytes = %d\n",bytes);
     bytes = MdcMatrixBlocks(bytes);
 
     MdcDebugPrint("DEBUG: bytes = %d\n",bytes);

     mbufr = malloc(bytes); 
     if (mbufr == NULL)
       return("ECAT7 Bad malloc matrix data buffer");

     error = mdc_mat_read_mat_data(fp,startblk,endblk,mbufr,ish.data_type);

     if (error) {
#if XSUPPORTED
       if (XMDC_MEDCON) {
         XMdcDisplayErr("ECAT7 Bad read matrix data");
       }else
#endif
       {
         MdcPrntWarn("ECAT7 Bad read matrix data");
       }
       err=MdcHandleTruncated(fi,img+1,MDC_YES);
       if(err != NULL) { MdcFree(mbufr); return(err); }
     }

     if (fi->truncated) break;


     /* bytes each image */
     bytes = fi->dim[1] * fi->dim[2] * MdcType2Bytes(type);

     for (slice=0; slice < fi->dim[3]; slice++, img++, plane++) {

#if XSUPPORTED
        if (XMDC_MEDCON) {
          pvalue += 1./(float)fi->number;
          XMdcUpdateProgressBar(NULL);
        }
#endif

        /* fill in IMG_DATA struct */
        id = &fi->image[img];
        id->width = fi->dim[1];
        id->height= fi->dim[2];
        id->bits  = bits;
        id->type  = type;
        id->quant_units = 1;
        id->quant_scale = 1;
        id->calibr_units= 1;
        id->calibr_fctr = 1;

        if (MDC_CALIBRATE == MDC_YES) {
          id->quant_scale  = ish.scale_factor;
          id->calibr_units = mh.calibration_units;
          id->calibr_fctr  = mh.ecat_calibration_factor;
        }else if (MDC_QUANTIFY == MDC_YES) {
          id->quant_scale  = ish.scale_factor;
        }

        id->pixel_xsize = ish.x_pixel_size * 10.0; /* mm */
        id->pixel_ysize = ish.y_pixel_size * 10.0; /* mm */
        id->slice_width = ish.z_pixel_size * 10.0; /* mm */
        id->slice_spacing = id->slice_width;
        id->frame_start = ish.frame_start_time;
        id->frame_duration = ish.frame_duration;
        strcpy(id->image_mod,"PT");

        MdcFillImgPos(fi,img,slice,ish.z_offset*10.);

        MdcFillImgOrient(fi,img);

        id->buf = MdcGetImgBuffer(bytes);
        if (id->buf == NULL) {
          MdcFree(mbufr);
          return("ECAT7 Bad malloc image buffer");
        }

        /* copy date */
        memcpy(id->buf, mbufr + (bytes*slice), bytes); 

     }

     MdcFree(mbufr);

  }

  /* set remaing FILEINFO data */
  id = &fi->image[0];
  fi->bits = id->bits;
  fi->type = id->type;
  fi->pixdim[0] = 3;
  fi->pixdim[1] = id->pixel_xsize;
  fi->pixdim[2] = id->pixel_ysize;
  fi->pixdim[3] = id->slice_width;

  switch (mh.file_type) {
    case MDC_ECAT7_FILE_TYPE_IMAGE8  :
    case MDC_ECAT7_FILE_TYPE_IMAGE16 :
    case MDC_ECAT7_FILE_TYPE_VOLUME8 :
    case MDC_ECAT7_FILE_TYPE_VOLUME16:
      fi->reconstructed = MDC_YES;
      if (ish.decay_corr_fctr > 1.0) fi->decay_corrected = MDC_YES;
      break; 
    default:
      fi->reconstructed = MDC_NO;
  }

  MdcFileClose(fi->ifp);

  if (fi->truncated) return("ECAT7 Truncated image file");
 
  return(NULL);

}

char *MdcWriteECAT7(FILEINFO *fi)
{
  return("ECAT7 Writing not yet supported");
}
