/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * filename: m-structs.c                                                   *
 *                                                                         *
 * UTIL C-source: Medical Image Conversion Utility                         *
 *                                                                         *
 * purpose      : structs handling functions                               *
 *                                                                         *
 * project      : (X)MedCon by Erik Nolf                                   *
 *                                                                         *
 * Functions    : MdcCheckFI()          - Check FILEINFO struct integrity  * 
 *                MdcGetStructID()      - Get IMG_DATA structs             *
 *                MdcGetStructSD()      - Get STATIC_DATA structs          *
 *                MdcGetStructGD()      - Get GATED_DATA structs           *
 *                MdcGetStructAD()      - Get ACQ_DATA structs             *
 *                MdcInitID()           - Initialize IMG_DATA structs      *
 *                MdcInitSD()           - Initialize STATIC_DATA strucs    *
 *                MdcInitGD()           - Initialize GATED_DATA structs    *
 *                MdcInitAD()           - Initialize ACQ_DATA structs      *
 *                MdcInitFI()           - Initialize FILEINFO struct       *
 *                MdcCopyID()           - Copy IMG_DATA    information     *
 *                MdcCopySD()           - Copy STATIC_DATA information     *
 *                MdcCopyGD()           - Copy GATED_DATA  information     *
 *                MdcCopyAD()           - Copy ACQ_DATA    information     *
 *                MdcCopyFI()           - Copy FILEINFO    information     *
 *                MdcFreeIDs()          - Free IMG_DATA structs            *
 *                MdcResetIDs()         - Reset IMG_DATA structs           *
 *                MdcCleanUpFI()        - Clean up FILEINFO struct         *
 *                                                                         *
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
/* $Id: m-structs.c,v 1.22 2003/06/10 22:51:15 enlf Exp $ 
 */

/*
   Copyright (C) 1997-2003 by Erik Nolf

   This program is free software; you can redistribute it and/or modify it
   under the terms of the GNU General Public License as published by the
   Free Software Foundation; either version 2, or (at your option) any later
   version.

   This program is distributed in the hope that it will be useful, but
   WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General
   Public License for more details.

   You should have received a copy of the GNU General Public License along
   with this program; if not, write to the Free Software Foundation, Inc.,
   59 Place - Suite 330, Boston, MA 02111-1307, USA.  */
 
/****************************************************************************
                              H E A D E R S
****************************************************************************/

#include "medcon.h"

/****************************************************************************
                            F U N C T I O N S
****************************************************************************/

char *MdcCheckFI(FILEINFO *fi)
{
  Uint32 i, t;

  /* check fi->dim[] values */ 
  if (fi->dim[0] <= 2 ) {
    sprintf(mdcbufr,"Internal ## fi->dim[0]=%d",fi->dim[0]);
    return(mdcbufr);
  }else{
    for (i=1; i<=fi->dim[0]; i++) {
       if (fi->dim[i] <= 0 ) {
         sprintf(mdcbufr,"Internal ## fi->dim[%d]=%d",i,fi->dim[i]);
         return(mdcbufr);
       }
    }
  }

  /* all fi->dim[] are 1-based, even unused */
  for (i=0; i<MDC_MAX_DIMS; i++) if (fi->dim[i] <= 0)
     return("Internal ## Dangerous negative fi->dim values");


  /* check fi->number value */
  for (i=1, t=3; t <= fi->dim[0]; t++) {
     i*=fi->dim[t];
  }
  if (fi->number != i) return("Internal ## Improper fi->dim values");

  return(NULL);

}

/* returns 1 on success and 0 on failure */
int  MdcGetStructID(FILEINFO *fi, Uint32 number)
{
  Uint32 i, begin=number; 

  /* bad request */
  if (number == 0) return(MDC_NO);

  /* allocate structs */
  if (fi->image == NULL) {
    /* fresh allocation */
    fi->image=(IMG_DATA *)malloc(sizeof(IMG_DATA)*number); 
    begin = 0; 
  }else if (number != fi->number) {
    /* reallocation */
    fi->image=(IMG_DATA *)realloc(fi->image,sizeof(IMG_DATA)*number);
    begin = number > fi->number ? fi->number : number;
  }

  if (fi->image == NULL) return(MDC_NO); 

  /* initialize new structs */
  for (i=begin; i<number; i++) MdcInitID(&fi->image[i]);  

  /* set new number */
  fi->number = number;

  return(MDC_YES);
}

/* returns 1 on success and 0 on failure        */
int MdcGetStructSD(FILEINFO *fi)
{
  Uint32 i;

  for (i=0; i<fi->number; i++) {
     fi->image[i].sdata = (STATIC_DATA *)malloc(sizeof(STATIC_DATA));
     if (fi->image[i].sdata == NULL) return(MDC_NO);
     MdcInitSD(fi->image[i].sdata);
  }

  return(MDC_YES);
}

/* returns 1 on success and 0 on failure        */
int MdcGetStructGD(FILEINFO *fi)
{
  Uint32 i;

  if ((fi->gatednr > 0) && (fi->gdata == NULL)) {
    fi->gdata = (GATED_DATA *)malloc(sizeof(GATED_DATA)*fi->gatednr);
    if (fi->gdata == NULL) return MDC_NO;

    for (i=0; i<fi->gatednr; i++) MdcInitGD(&fi->gdata[i]);

  }

  if (fi->gdata != NULL) return(MDC_YES); /* (already) allocated */

  return(MDC_NO);

}

/* returns 1 on success and 0 on failure        */
int MdcGetStructAD(FILEINFO *fi)
{
  Uint32 i;

  if ((fi->acqnr > 0) && (fi->acqdata == NULL)) {
    /* allocate one or an array of items */
    fi->acqdata = (ACQ_DATA *)malloc(sizeof(ACQ_DATA)*fi->acqnr);
    if (fi->acqdata == NULL) return MDC_NO;

    for (i=0; i<fi->acqnr; i++) MdcInitAD(&fi->acqdata[i]);

  }else if ((fi->acqnr > 1) && (fi->acqdata != NULL)) {
    /* allocate more then one default item */
    fi->acqdata = (ACQ_DATA *)realloc(fi->acqdata,sizeof(ACQ_DATA)*fi->acqnr);
    if (fi->acqdata == NULL) return MDC_NO;

    for (i=1; i<fi->acqnr; i++) MdcInitAD(&fi->acqdata[i]);

  }

  if (fi->acqdata != NULL) return(MDC_YES); /* (already) allocated */

  return(MDC_NO);

}

void MdcInitID(IMG_DATA *id)
{
  int i;

  memset(id,'\0',sizeof(IMG_DATA));

  id->rescaled = MDC_NO;
  id->quant_scale = 1.;
  id->calibr_fctr = 1.;
  id->rescale_slope = 1.;
  id->rescale_intercept = 0.;
  id->quant_units = id->calibr_units = 1;
  id->buf = NULL;

  id->pixel_xsize = 1.;
  id->pixel_ysize = 1.;
  id->slice_width = 1.;

  strcpy(id->image_mod,"NM");

  for(i=0;i<3;i++) { id->image_pos_dev[i]=0.;    id->image_pos_pat[i]=0.;}
  for(i=0;i<6;i++) { id->image_orient_dev[i]=0.; id->image_orient_pat[i]=0.;}

  id->slice_spacing = 0.; /* = slice_width; when not found no gap */ 

  id->ct_zoom_fctr  = 1.;
 
  id->sdata = NULL;
 
  id->plugb = NULL;
}

void MdcInitSD(STATIC_DATA *sd)
{
  strcpy(sd->label,"Unknown");
  sd->total_counts      = 0.;
  sd->image_duration    = 0.;
  sd->start_time_hour   = 0;
  sd->start_time_minute = 0;
  sd->start_time_second = 0;
}

void MdcInitGD(GATED_DATA *gd) 
{
  if (gd != NULL) {
    gd->gspect_nesting = MDC_GSPECT_NESTING_GATED;
    gd->nr_projections = 0.0;
    gd->extent_rotation = 0.0;
    gd->study_duration = 0.0;
    gd->image_duration = 0.0;
    gd->time_per_proj  = 0.0;
    gd->window_low = 0.0;
    gd->window_high= 0.0;
    gd->cycles_observed = 0.0;
    gd->cycles_acquired = 0.0;
    
  }

}

void MdcInitAD(ACQ_DATA *acq)
{
  if (acq != NULL) {
     acq->rotation_direction = MDC_ROTATION_CW;
     acq->detector_motion = MDC_MOTION_STEP;
     acq->rotation_offset = 0.;
     acq->radial_position = 0.;
     acq->angle_start = 0.;
     acq->angle_step = 0.; 
     acq->scan_arc = 360.;
  }
}

void MdcInitFI(FILEINFO *fi, char *path)
{  
   fi->ifp = NULL; fi->ofp = NULL;
   fi->idir = NULL; fi->ifname = NULL;
   fi->odir = NULL; fi->ofname = NULL;
   fi->image = NULL;
   fi->format = MDC_FRMT_NONE;
   fi->diff_type = fi->diff_size = MDC_NO;
   fi->rawconv = MDC_NO;
   fi->endian = MDC_UNKNOWN;
   fi->compression = MDC_NO;
   fi->truncated=MDC_NO;
   fi->number = 0;
   fi->mwidth=fi->mheight=0;
   fi->bits = 8; fi->type = BIT8_U;
   fi->ifname = fi->ipath;
   memset(fi->ipath,'\0',MDC_MAX_PATH);
   strncpy(fi->ipath,path,MDC_MAX_PATH);
   fi->ofname = fi->opath;
   memset(fi->opath,'\0',MDC_MAX_PATH);
   fi->study_date_day   = 0;
   fi->study_date_month = 0;
   fi->study_date_year  = 0;
   fi->study_time_hour  = 0;
   fi->study_time_minute= 0;
   fi->study_time_second= 0;

   fi->nr_series      = -1;
   fi->nr_acquisition = -1;
   fi->nr_instance    = -1;

   fi->decay_corrected   = MDC_NO;
   fi->flood_corrected   = MDC_NO;
   fi->acquisition_type  = MDC_ACQUISITION_UNKNOWN;
   fi->reconstructed     = MDC_YES;

   fi->slice_projection = MDC_UNKNOWN;
   fi->pat_slice_orient = MDC_UNKNOWN;

   strcpy(fi->pat_pos,"Unknown");
   strcpy(fi->pat_orient,"Unknown");
   strcpy(fi->recon_method,"Unknown");
   strcpy(fi->patient_name,"Unknown");
   strcpy(fi->patient_id,"Unknown");
   strcpy(fi->patient_sex,"Unknown");
   strcpy(fi->patient_dob,"0000:00:00");
   strcpy(fi->study_descr,"Unknown");
   strcpy(fi->study_id,"Unknown");
   strcpy(fi->series_descr,"Unknown");
   strcpy(fi->radiopharma,"Unknown");
   strcpy(fi->filter_type,"Unknown");
   strcpy(fi->organ_code,"Unknown");
   strcpy(fi->isotope_code,"Unknown");
 
   fi->patient_weight   = 0.;
   fi->isotope_halflife = 0.;
   fi->injected_dose    = 0.;
   fi->gantry_tilt      = 0.;

   fi->dim[0] = 3;
   fi->dim[1] = 1;
   fi->dim[2] = 1;
   fi->dim[3] = 1;
   fi->dim[4] = 1;
   fi->dim[5] = 1;
   fi->dim[6] = 1;
   fi->dim[7] = 1;
   fi->pixdim[0] = 3.;
   fi->pixdim[1] = 1.;
   fi->pixdim[2] = 1.;
   fi->pixdim[3] = 1.;
   fi->pixdim[4] = 1.;
   fi->pixdim[5] = 1.;
   fi->pixdim[6] = 1.;
   fi->pixdim[7] = 1.;


   fi->map = MDC_MAP_GRAY;
   MdcGetColorMap(fi->map,fi->palette);
   fi->comment = NULL;
   fi->comm_length = 0;
   fi->glmin = fi->glmax = fi->qglmin = fi->qglmax = 0.;

   fi->gatednr = 0; fi->gdata = NULL;

   fi->acqnr = 0; fi->acqdata = NULL;

   fi->mod = NULL;

   fi->pluga = NULL;


}

char *MdcCopySD(STATIC_DATA *dest, STATIC_DATA *src)
{
  strncpy(dest->label,src->label,MDC_MAXSTR);
  dest->total_counts   = src->total_counts;
  dest->image_duration = src->image_duration;
  dest->start_time_hour   = src->start_time_hour;
  dest->start_time_minute = src->start_time_minute;
  dest->start_time_second = src->start_time_second;

  return(NULL);

}

char *MdcCopyGD(GATED_DATA *dest, GATED_DATA *src)
{
  dest->gspect_nesting  = src->gspect_nesting;
  dest->nr_projections  = src->nr_projections;
  dest->extent_rotation = src->extent_rotation;
  dest->study_duration  = src->study_duration;
  dest->image_duration  = src->image_duration;
  dest->time_per_proj   = src->time_per_proj;
  dest->window_low      = src->window_low;
  dest->window_high     = src->window_high;
  dest->cycles_observed = src->cycles_observed;
  dest->cycles_acquired = src->cycles_acquired;

  return(NULL);
}

char *MdcCopyAD(ACQ_DATA *dest, ACQ_DATA *src)
{
  dest->rotation_direction = src->rotation_direction;
  dest->detector_motion    = src->detector_motion;
  dest->rotation_offset    = src->rotation_offset;
  dest->radial_position    = src->radial_position;
  dest->angle_start        = src->angle_start;
  dest->angle_step         = src->angle_step;
  dest->scan_arc           = src->scan_arc;

  return(NULL);
}

char *MdcCopyID(IMG_DATA *dest, IMG_DATA *src, int COPY_IMAGE)
{
  Uint32 i, w, h, b, size;

  dest->width = src->width;    dest->height = src->height;
  dest->bits  = src->bits;     dest->type   = src->type;
  dest->flags = src->flags;
  dest->min   = src->min;      dest->max    = src->max;
  dest->qmin  = src->qmin;     dest->qmax   = src->qmax;
  dest->fmin  = src->fmin;     dest->fmax   = src->fmax;
  dest->qfmin = src->qfmin;    dest->qfmax  = src->qfmax;

  if (COPY_IMAGE == MDC_YES) {

    dest->rescale_slope = src->rescale_slope;
    dest->rescale_intercept = src->rescale_intercept; 

    w = dest->width; h = dest->height; b = MdcType2Bytes(dest->type);
    size = w * h * b;
    dest->buf = malloc(size);
    if (dest->buf == NULL)  return("Failed to copy image buffer");
    memcpy(dest->buf,src->buf,size);

    dest->rescaled      = src->rescaled;
    dest->rescaled_min  = src->rescaled_min;
    dest->rescaled_max  = src->rescaled_max;
    dest->rescaled_fctr = src->rescaled_fctr;
    dest->rescaled_slope= src->rescaled_slope;
    dest->rescaled_intercept = src->rescaled_intercept;

    dest->quant_scale   = src->quant_scale;
    dest->calibr_fctr   = src->calibr_fctr;

  }else{
    dest->rescale_slope = 1.;
    dest->rescale_intercept = 0.;
    dest->buf = NULL;
    dest->rescaled = MDC_NO;
    dest->rescaled_min  = 0.;
    dest->rescaled_max  = 0.;
    dest->rescaled_fctr = 1.;
    dest->rescaled_slope= 1.;
    dest->rescaled_intercept = 0.;

    dest->quant_scale = 1.;
    dest->calibr_fctr = 1.;
  }

  dest->quant_units  = src->quant_units;
  dest->calibr_units = src->calibr_units;

  dest->pixel_xsize  = src->pixel_xsize;
  dest->pixel_ysize  = src->pixel_ysize;
  dest->slice_width  = src->slice_width;
  dest->frame_start  = src->frame_start;
  dest->frame_duration   = src->frame_duration;
  dest->recon_scale      = src->recon_scale;

  strncpy(dest->image_mod,src->image_mod,3);

  for (i=0; i<3; i++) dest->image_pos_dev[i] = src->image_pos_dev[i];
  for (i=0; i<6; i++) dest->image_orient_dev[i] = src->image_orient_dev[i];
  for (i=0; i<3; i++) dest->image_pos_pat[i] = src->image_pos_pat[i];
  for (i=0; i<6; i++) dest->image_orient_pat[i] = src->image_orient_pat[i];

  dest->slice_spacing = src->slice_spacing;
  dest->ct_zoom_fctr  = src->ct_zoom_fctr;

  /* static data */
  if (src->sdata != NULL) {
    dest->sdata = (STATIC_DATA *)malloc(sizeof(STATIC_DATA));
    if (dest->sdata == NULL) return("Failed to copy static data struct");
    MdcCopySD(dest->sdata,src->sdata);
  }
 
  return(NULL);
}

char *MdcCopyFI(FILEINFO *dest, FILEINFO *src, int COPY_IMAGES)
{
  char *msg=NULL;
  int i;

  MdcInitFI(dest,src->ifname);

  dest->ifp = src->ifp;
  dest->ofp = src->ofp;

  strncpy(dest->ipath,src->ipath,MDC_MAX_PATH);
  strncpy(dest->opath,src->opath,MDC_MAX_PATH);

  dest->format  = src->format;
  dest->rawconv = src->rawconv;
  dest->endian  = src->endian;
  dest->compression = src->compression;
  dest->truncated   = src->truncated;
  dest->diff_type   = src->diff_type;
  dest->diff_size   = src->diff_size;
/*dest->number      = src->number;*/     /* just see later  */
  dest->mwidth      = src->mwidth;
  dest->mheight     = src->mheight;
  dest->bits        = src->bits;
  dest->type        = src->type;

  for (i=0; i<MDC_MAX_DIMS; i++) dest->dim[i]    = src->dim[i];
  for (i=0; i<MDC_MAX_DIMS; i++) dest->pixdim[i] = src->pixdim[i]; 

  dest->glmin       = src->glmin;
  dest->glmax       = src->glmax;
  dest->qglmin      = src->qglmin;
  dest->qglmax      = src->qglmax;

  dest->slice_projection = src->slice_projection;
  dest->pat_slice_orient = src->pat_slice_orient;

  strncpy(dest->pat_pos,src->pat_pos,MDC_MAXSTR);
  strncpy(dest->pat_orient,src->pat_orient,MDC_MAXSTR);
  strncpy(dest->patient_sex,src->patient_sex,MDC_MAXSTR);
  strncpy(dest->patient_name,src->patient_name,MDC_MAXSTR);
  strncpy(dest->patient_id,src->patient_id,MDC_MAXSTR);
  strncpy(dest->patient_dob,src->patient_dob,MDC_MAXSTR);
  strncpy(dest->study_descr,src->study_descr,MDC_MAXSTR);
  strncpy(dest->study_id,src->study_id,MDC_MAXSTR);

  dest->study_date_day   = src->study_date_day;
  dest->study_date_month = src->study_date_month;
  dest->study_date_year  = src->study_date_year;
  dest->study_time_hour  = src->study_time_hour;
  dest->study_time_minute= src->study_time_minute;
  dest->study_time_second= src->study_time_second;
  dest->nr_series        = src->nr_series;
  dest->nr_acquisition   = src->nr_acquisition;
  dest->nr_instance      = src->nr_instance;
  dest->acquisition_type = src->acquisition_type;
  dest->decay_corrected  = src->decay_corrected;
  dest->flood_corrected  = src->flood_corrected;
  dest->reconstructed    = src->reconstructed;

  strncpy(dest->recon_method,src->recon_method,MDC_MAXSTR);
  strncpy(dest->series_descr,src->series_descr,MDC_MAXSTR);
  strncpy(dest->radiopharma,src->radiopharma,MDC_MAXSTR);
  strncpy(dest->filter_type,src->filter_type,MDC_MAXSTR);
  strncpy(dest->organ_code,src->organ_code,MDC_MAXSTR);
  strncpy(dest->isotope_code,src->isotope_code,MDC_MAXSTR);

  dest->patient_weight   = src->patient_weight;
  dest->isotope_halflife = src->isotope_halflife;
  dest->gantry_tilt      = src->gantry_tilt;
  dest->injected_dose    = src->injected_dose;

  dest->map = src->map;
  memcpy(dest->palette,src->palette,768);

  /* copy comment */
  if (src->comm_length > 0) {
    dest->comment = malloc(src->comm_length);
    if (dest->comment == NULL) {
      /* bad, but don't fail on some comment */
      dest->comm_length = 0;
    }else{
      dest->comm_length = src->comm_length;
      memcpy(dest->comment,src->comment,dest->comm_length);
    }
  }

  /* copy ACQ_DATA structs */
  if (src->acqnr > 0 && src->acqdata != NULL) {
    dest->acqnr = src->acqnr;
    dest->acqdata = (ACQ_DATA *)malloc(dest->acqnr * sizeof(ACQ_DATA));
    if (dest->acqdata == NULL) return("Failed to create ACQ_DATA structs");
    for (i=0; i<dest->acqnr; i++) {
       msg = MdcCopyAD(&dest->acqdata[i],&src->acqdata[i]);
       if (msg != NULL) return(msg);
    }
  }else{
    dest->acqnr = 0; dest->acqdata = NULL;
  }

  /* copy GATED_DATA structs */
  if (src->gatednr > 0 && src->gdata != NULL) {
    dest->gatednr = src->gatednr;
    dest->gdata = (GATED_DATA *)malloc(dest->gatednr * sizeof(GATED_DATA));
    if (dest->gdata == NULL) return("Failed to create GATED_DATA structs");
    for (i=0; i<dest->gatednr; i++) {
       msg = MdcCopyGD(&dest->gdata[i],&src->gdata[i]);
       if (msg != NULL) return(msg);
    }
  }else{
    dest->gatednr = 0; dest->gdata = NULL;
  }

  /* copy IMG_DATA structs */
  if ((COPY_IMAGES == MDC_YES) && (src->number > 0) && (src->image != NULL)) {
    dest->number = src->number;
    dest->image = (IMG_DATA *)malloc(dest->number * sizeof(IMG_DATA));
    if (dest->image == NULL) return("Failed to create IMG_DATA structs");
    for (i=0; i<dest->number; i++) {
       msg = MdcCopyID(&dest->image[i],&src->image[i],MDC_YES);
       if (msg != NULL) return(msg);
    } 
  }else{
    dest->number = 0; dest->image = NULL;
  }

  return(NULL);
}

void MdcFreeMODs(FILEINFO *fi)
{
  MOD_INFO *mod = fi->mod;

  if (mod != NULL) {
    MdcFree(mod->xa_info);
    MdcFree(mod->mr_info);

    MdcFree(fi->mod);
  }
  fi->mod = NULL;
}

void MdcFreeIDs(FILEINFO *fi)
{ 
  IMG_DATA *id=NULL;
  Uint32 i;
 
  if ( fi->image != NULL ) {
    for ( i=0; i<fi->number; i++) {
       id = (IMG_DATA *)&fi->image[i];
       MdcFree(id->buf);
       MdcFree(id->sdata);
       MdcFree(id->plugb);
    }
    MdcFree(fi->image);
  }
  fi->image = NULL;
}

void MdcResetIDs(FILEINFO *fi)
{
  Uint32 i;

  for (i=0; i<fi->number; i++)  {
     fi->image[i].rescaled = MDC_NO;
     fi->image[i].rescaled_max  = 0.;
     fi->image[i].rescaled_min  = 0.;
     fi->image[i].rescaled_fctr = 1.;
     fi->image[i].rescaled_slope= 1.;
     fi->image[i].rescaled_intercept = 0.;
  }

}

void MdcCleanUpFI(FILEINFO  *fi)
{
  if (fi->acqnr   > 0) MdcFree(fi->acqdata);
  if (fi->gatednr > 0) MdcFree(fi->gdata);
  if (fi->comm_length > 0) MdcFree(fi->comment);
  MdcFreeIDs(fi);

  MdcFileClose(fi->ifp);
  MdcFileClose(fi->ofp);

  if (fi->mod != NULL) MdcFreeMODs(fi);

  if (fi->pluga != NULL) MdcFree(fi->pluga);

}

